/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.OperationHistoryFactory;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.operations.IWorkbenchOperationSupport;
import org.eclipse.ui.views.properties.PropertySheet;
import org.eclipse.ui.views.properties.PropertySheetPage;

import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.actions.cdm.SaveTaxonAction;
import eu.etaxonomy.taxeditor.datasource.CdmTransactionController;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.name.CdmParserController;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;
import eu.etaxonomy.taxeditor.model.CdmSessionDataRepository;
import eu.etaxonomy.taxeditor.model.NameEditorInput;
import eu.etaxonomy.taxeditor.navigation.RecentNamesView;
import eu.etaxonomy.taxeditor.navigation.TaxonomicTreeView;
import eu.etaxonomy.taxeditor.navigation.TaxonomicTreeViewer;
import eu.etaxonomy.taxeditor.propertysheet.CustomSortPropertySheetEntry;

/**
 * A collection of useful methods related to the UI.
 * 
 * @author p.ciardelli
 * @created 27.05.2008
 * @version 1.0
 */
public class UiUtil {
	private static final Logger logger = Logger.getLogger(UiUtil.class);

	public static Set<Feature> preferredFeatureSet;
	public static Set<Rank> preferredRankSet;
	private static IViewPart treeView;

	private static IOperationHistory operationHistory;
	
	public static IViewPart getPropertySheet() {
		for (IViewReference reference : getActivePage().getViewReferences()) {
			if (reference.getId().equals(IPageLayout.ID_PROP_SHEET)) {
				logger.warn(reference.getView(false).getSite().getPart().getTitle());
				return reference.getView(false);
			}
		}
		return null;
	}
	
	/**
	 * By default, property sheet has buttons in the toolbar for 
	 * "Show advanced properties" and "Show categories".
	 * <p>
	 * This is confusing for the user, hence a method to remove them
	 * until such time as advanced properties or categories are implemented.
	 */
	public static void hidePropertySheetToolbar() {
		PropertySheet propertySheet = (PropertySheet) getPropertySheet();
		IActionBars actionBars = propertySheet.getViewSite().getActionBars();
		actionBars.getToolBarManager().removeAll();
		actionBars.getMenuManager().removeAll();
	}
	
	/**
	 * The property sheet listener ensures only property sheets
	 * 	with data cause the Property Sheet to be updated.
	 */
	public static void addPropertySheetInputListener() {
		IViewPart propertySheet = getPropertySheet();
//		propertySheet.get
		PropertySheet ps = (PropertySheet) propertySheet;
//		ps.addPartPropertyListener(listener)
//		ps.addPropertyListener(l)
	}
	
	public static void saveAll(){
		setIsSaving(true);
		
		// Get all open windows
		for (IEditorPart taxonEditor : getOpenTaxonEditors()) {

			// Save the dirty ones
			if (taxonEditor.isDirty()) {
				
				
				IEditorInput input = taxonEditor.getEditorInput();
				if (input.getAdapter(Taxon.class) != null) {
					Taxon taxon = (Taxon) input.getAdapter(Taxon.class);
					CdmSessionDataRepository.getDefault().saveTaxon(taxon);
					if (taxonEditor instanceof MultiPageTaxonEditor) {
						((MultiPageTaxonEditor) taxonEditor).setDirtyExtern(false);
					}
				}
			}
		}

		// Commit the transaction	
		CdmTransactionController.commitTransaction();
	
		// Force library objects to be associated with new transaction
		CdmSessionDataRepository.getDefault().clearNonTaxonData();
		
		// Start a new transaction
		CdmTransactionController.startTransaction();
		
		// Put all open taxa in the new transaction
		CdmTransactionController.addSessionTaxaToTransaction();
		
		setIsSaving(false);
		// TODO: delete undoHistory
	}
	
	//public void deleteTaxon()
	
	/**
	 * @param input
	 * @return
	 * @throws PartInitException
	 */
	public static IEditorPart getEditorByInput(IEditorInput input)
			throws PartInitException {
		for (IEditorReference reference : getActivePage().getEditorReferences()) {
			if (reference.getEditorInput().equals(input)) {
				IEditorPart editor = reference.getEditor(false);
				return editor;
			}
		}
		return null;
	}
	
	/**
	 * Returns a set of all currently open 
	 * <code>MultiPageTaxonEditor</code>s.
	 * 
	 * @return
	 */
	public static Set<IEditorPart> getOpenTaxonEditors() {
		
		Set<IEditorPart> taxonEditors = new HashSet<IEditorPart>();
		
		for (IEditorReference reference : getActivePage().getEditorReferences()) {
			IEditorPart editor = reference.getEditor(false);
			if (editor instanceof MultiPageTaxonEditor) {
				taxonEditors.add(editor);
			}
		}
		return taxonEditors;
	}

	public static boolean closeOpenTaxonEditors() {
		for (IEditorPart editor : getOpenTaxonEditors()) {
			if (!getActivePage().closeEditor(editor, true)) {
				return false;
			}
		}
		return true;
	}
	
	public static IEditorPart getEditorByTaxon(Taxon taxon)
			throws PartInitException {
		IEditorInput input = new NameEditorInput(taxon);
		return getEditorByInput(input);
	}
	
	/**
	 * @return
	 */
	public static IWorkbenchPage getActivePage() {
		return TaxEditorPlugin.getDefault().getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
	}

	/**
	 * @return
	 */
	public static Shell getShell() {
		return TaxEditorPlugin.getDefault().getWorkbench()
				.getActiveWorkbenchWindow().getShell();
	}

	/**
	 * @param input
	 * @param editorId
	 * @return 
	 * @throws PartInitException
	 */
	public static IEditorPart openEditor(IEditorInput input, String editorId)
			throws PartInitException {
		return getActivePage().openEditor(input, editorId);
	}
	
	/**
	 * Open an emtpy taxon editor
	 * 
	 * @return
	 * @throws PartInitException
	 */
	public static IEditorPart openTaxonEditor() 
			throws PartInitException{
		return openTaxonEditor(null);
	}
	
	/** 
	 * Open a taxon editor for the given taxon
	 * 
	 * @param taxon
	 * @return
	 * @throws PartInitException
	 */
	public static IEditorPart openTaxonEditor(Taxon taxon)
			throws PartInitException{
		if(taxon == null){
			TaxonNameBase name = CdmParserController.parseFullReference("", null, null);
			name.setFullTitleCache("", false);
			name.setTitleCache("", false);
			taxon = Taxon.NewInstance(name, 
					CdmSessionDataRepository.getDefault().getDefaultSec());
		}else{
			// If this taxon is not visible in the tree, open node
			TaxonomicTreeViewer treeViewer = getTreeViewer(); 
			if (treeViewer != null) {
				treeViewer.revealTaxon(taxon);
			}
			
			// Add to recent names list
			RecentNamesView.addRecentName(taxon);
		}
		
		IEditorInput input = new NameEditorInput(taxon);
		return openEditor(input, MultiPageTaxonEditor.ID);
	}

	/**
	 * @param taxon
	 * @param save
	 * @throws PartInitException
	 */
	public static void closeEditor(Taxon taxon, boolean save)
			throws PartInitException {
		IEditorPart editor = getEditorByTaxon(taxon);
		closeEditor(editor, save);
	}
	
	/**
	 * @param input
	 * @param save
	 * @throws PartInitException
	 */
	public static void closeEditor(IEditorInput input, boolean save)
			throws PartInitException {
		IEditorPart editor = getEditorByInput(input);
		closeEditor(editor, save);
	}

	/**
	 * @param editor
	 * @param save
	 * @throws PartInitException
	 */
	public static void closeEditor(IEditorPart editor, boolean save)
			throws PartInitException {
		if (editor != null) {
			getActivePage().closeEditor(editor, save);
		}
	}

	public static IViewPart getViewById(String id) {
		return TaxEditorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().
				getActivePage().findView(id);
	}
	

	/**
	 * Returns the <code>TaxonomicTreeViewer</code> used for navigation. Not to 
	 * be confused with the window that contains it, <code>TaxonomicTreeView</code>. 
	 * 
	 * @return
	 */
	public static TaxonomicTreeViewer getTreeViewer() {
		TaxonomicTreeView view = getTreeView();
		if (view == null) {
			return null;
		}
		return view.getTreeViewer();
	}

	/**
	 * @see eu.etaxonomy.taxeditor.navigation.TaxonomicTreeView#createTreeViewer()
	 * 
	 * @return
	 */
	public static TaxonomicTreeViewer createTreeViewer() {
		TaxonomicTreeView view = getTreeView();
		if (view == null) {
			return null;
		}
		return view.createTreeViewer();
	}
	
	/**
	 * Returns the <code>TaxonomicTreeView</code> containing the 
	 * <code>TaxonomicTreeViewer</code> used for navigation. 
	 * 
	 * @return
	 */	
	public static TaxonomicTreeView getTreeView() {
		if (treeView == null) {
			treeView = getViewById(TaxonomicTreeView.ID);
			if (!(treeView instanceof TaxonomicTreeView)) {
				treeView = null;
			}
		}
		return (TaxonomicTreeView) treeView;
	}
	

	public static IWorkbenchOperationSupport getOperationSupport() {
		IWorkbench workbench = TaxEditorPlugin.getDefault().getWorkbench();
		return workbench.getOperationSupport();
	}
	
	public static IOperationHistory getOperationHistory() {
//		if (operationHistory == null) {
////			operationHistory = getOperationSupport().getOperationHistory();
//			operationHistory = OperationHistoryFactory.getOperationHistory();
//		}
//		return operationHistory;
		return getOperationSupport().getOperationHistory();
	}

	public static IUndoContext getWorkbenchUndoContext() {
		// TODO make this more specific than GLOBAL_UNDO_CONTEXT
//		return getOperationSupport().getUndoContext();
		return IOperationHistory.GLOBAL_UNDO_CONTEXT;
	}

	public static void doEditorSave(Taxon taxon, boolean confirm) {
		IEditorPart editor = null;
		try {
			editor = getEditorByTaxon(taxon);
		} catch (PartInitException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (editor != null) {
			getActivePage().saveEditor(editor, confirm);
		}
	}
	

//	public static EditorGroupComposite getMisappliedNameGroupComposite(
//			IManagedForm managedForm) {
//
//		// Iterate through parent's children until we find a composite which has a data field
//		//	for MISAPPLIED_NAME
//		Composite parent = managedForm.getForm().getBody();
//		for (Control groupComposite : parent.getChildren()) {
//			if (groupComposite instanceof EditorGroupComposite) {
//				if (groupComposite.getData(ITaxEditorConstants.MISAPPLIED_NAME) != null) {
//					return (EditorGroupComposite) groupComposite;
//				}
//			}
//		}
//		return null;
//	}

//	public static EditorGroupComposite createMisappliedNameGroupComposite(
//			IManagedForm managedForm) {
//		
//		Composite parent = managedForm.getForm().getBody();
//		
//		EditorGroupComposite composite = new EditorGroupComposite(parent, managedForm);
//		composite.setData(ITaxEditorConstants.MISAPPLIED_NAME,"");
//		new CompositeBorderDecorator(composite, managedForm);
//		return composite;
//	}
	
	public static void setStatusMessage(String msg) {
		getPropertySheet().getViewSite().getActionBars().getStatusLineManager().setMessage(msg); 
	}

	public static void setPropertySheetTree(Tree tree) {
		TaxEditorPlugin.getDefault().setPropertySheetTree(tree);		
	}
	
	public static Tree getPropertySheetTree() {
		return TaxEditorPlugin.getDefault().getPropertySheetTree();		
	}

	public static void setPropertySheetPage(PropertySheetPage page) {
		TaxEditorPlugin.getDefault().setPropertySheetPage(page);
	}
	
	public static PropertySheetPage getPropertySheetPage() {
		return TaxEditorPlugin.getDefault().getPropertySheetPage();	
	}
	
	/**
	 * 		UiUtil.paintPropertySheetRow(P_DATEPUBLISHED, new Color(Display.getDefault(), WarningAnnotation.WARNING_RGB), true);
	 * 		UiUtil.unpaintPropertySheetRow(P_DATEPUBLISHED);
	 * 
	 * @param id
	 * @param color
	 * @param doPaintChildren
	 */
	public static void paintPropertySheetRow(String id, Color color, boolean doPaintChildren) {
		
		// Catch null property sheet name
		if (id == null) {
			return;
		}
		
		// Catch uninit'ed property sheet tree
		if (getPropertySheetTree() == null) {
			return;
		}
		
		paintPropertySheetRow(id, color, doPaintChildren, getPropertySheetTree());
	}
	
	private static void paintPropertySheetRow(String id, Color color, boolean doPaintChildren, Object treeOrItem) {
		
		// Init items w zero-length array
		TreeItem[] items = new TreeItem[]{};
		
		// Get child items depending to class
		if (treeOrItem instanceof Tree) {
			items = ((Tree) treeOrItem).getItems();
		}
		if (treeOrItem instanceof TreeItem) {
			items = ((TreeItem) treeOrItem).getItems();
		}
		
		// If array hasn't been populated by the above, return
		if (items.length == 0) {
			return;
		}

		// Prop. sheet id's take the form "01:xxxx" for sorting - truncate
		id = CustomSortPropertySheetEntry.truncateDisplayName(id);
		
		// Iterate through child items
		for (TreeItem item : items) {

			// Item found, paint it
			if (id.equals(item.getText())) {
				paintItem(item, color, doPaintChildren);
				return;
			}			

			// Recursively search for item to paint in child items
			if (item.getItemCount() > 0) {
				paintPropertySheetRow(id, color, doPaintChildren, item);
			}
		}	
	}
	
	/**
	 * Note: children are only painted if submenu has already been created, i.e. opened once.
	 * 
	 * @param item
	 * @param color
	 * @param doPaintChildren
	 */
	private static void paintItem(TreeItem item, Color color, boolean doPaintChildren) {
		
		// Paint the item
		item.setBackground(color);
		
		// Recursively paint child items if requested
		if (doPaintChildren) {
			for (TreeItem childItem : item.getItems()) {
				paintItem(childItem, color, doPaintChildren);
			}
		}
	}
	
	public static void unpaintPropertySheetRow(String id) {
		
		// Catch uninit'ed property sheet tree
		if (getPropertySheetTree() == null) {
			return;
		}
		
		// Get tree's background color to "unpaint"
		Color color = getPropertySheetTree().getBackground();
		
		paintPropertySheetRow(id, color, true);
	}

	/**
	 * Get the name out of the session's name relations vocabulary, not directly
	 * from the object. 
	 * 
	 * @param type
	 * @return
	 */
	public static String getNameRelationLabelType(NameRelationshipType type) {
		SortedSet<NameRelationshipType> vocab = 
				CdmSessionDataRepository.getDefault().getNameRelationshipTypes();
		for (NameRelationshipType type1 : vocab) {
			if (type1.equals(type)) {
				return type1.getLabel();
			}
		}
		return "";
	}

	static boolean isSaving = false;

	private static IStatusLineManager statusLineManager;

	private static Map<Taxon, TaxonNameEditor> taxonNameEditors;

	public static void setIsSaving(boolean isSavin) {
		isSaving = isSavin;
	}
	
	public static boolean getIsSaving() {
		return isSaving;
	}

	public static void setStatusLineManager(IStatusLineManager manager) {
		statusLineManager = manager;
	}

	public static void setStatusLine(String message) {
		statusLineManager.setMessage(message);
	}
	
	public static IProgressMonitor getStatusLineProgressMonitor() {
		statusLineManager.setCancelEnabled(false);
		return statusLineManager.getProgressMonitor();
	}

	public static void addTaxonNameEditor(
			Taxon taxon, TaxonNameEditor taxonNameEditor) {
		if (taxonNameEditors == null) {
			taxonNameEditors = new HashMap<Taxon, TaxonNameEditor>();
		}
		taxonNameEditors.put(taxon, taxonNameEditor);
	}

	public static TaxonNameEditor getTaxonNameEditor(Taxon taxon) {
		if (taxonNameEditors == null) {
			return null;
		}
		return taxonNameEditors.get(taxon);
	}

	public static IUndoContext getTaxonNameEditorUndoContext(Taxon taxon) {
		// TODO make this taxon name editor specific
//		return getTaxonNameEditor(taxon).getUndoContext();
		return IOperationHistory.GLOBAL_UNDO_CONTEXT;
	}
}