/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.actions.ui;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.IPropertyChangeListener;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.actions.cdm.ChangeSynonymHomotypicGroupAction;
import eu.etaxonomy.taxeditor.actions.cdm.ChangeTaxonRelationToSynonymAction;
import eu.etaxonomy.taxeditor.editor.EditorGroupComposite;
import eu.etaxonomy.taxeditor.editor.name.NameComposite;
import eu.etaxonomy.taxeditor.model.CdmUtil;

/**
 * Reassigns the synonym attached to the NameComposite's data object to the
 * HomotypicalGroup specified in the EditorGroupComposite's data object.
 * 
 * If this is successful, new Synonym is attached to the NameComposite's data
 * object, and NameComposite is cosmetically transformed according to its
 * synonymy type.
 * 
 * @author p.ciardelli
 * @created 02.06.2008
 * @version 1.0
 */
public class AdaptCompositeToGroupAction extends Action {
	private static final Logger logger = Logger
			.getLogger(AdaptCompositeToGroupAction.class);

	private static String text = "Move name composite to new homotypic group";
	private ImageDescriptor image = null;

	private IUndoableOperation operation;

	private NameComposite composite;
	// private Synonym synonym = null;
	private TaxonBase taxonBase = null;
	private HomotypicalGroup newHomotypicGroup;
	private Taxon taxon;

	private AdaptCompositeToGroupAction() {
		super(text);
		setImageDescriptor(image);
	}

	public AdaptCompositeToGroupAction(final NameComposite composite,
			EditorGroupComposite groupComposite) {
		this();

		if (composite.getData() instanceof TaxonBase) {
			this.taxonBase = (TaxonBase) composite.getData();
		} else {
			throw new IllegalArgumentException(
					"This action requires a NameComposite with a TaxonBase (Synonym or Taxon) in its data field.");
		}

		if (groupComposite.getData() instanceof HomotypicalGroup) {
			this.newHomotypicGroup = (HomotypicalGroup) groupComposite
					.getData();
		} else {
			throw new IllegalArgumentException(
					"This action requires a EditorGroupComposite with a HomotypicalGroup in its data field.");
		}

		this.composite = composite;

		if (groupComposite.getParent().getData() instanceof Taxon) {
			this.taxon = (Taxon) groupComposite.getParent().getData();
		} else {
			throw new IllegalArgumentException(
					"This action requires a EditorGroupComposite whose parent has a Taxon in its data field.");
		}

		operation = new ChangeHomotypicGroupOperation();

	}

	public void run() {
		IOperationHistory operationHistory = UiUtil.getOperationHistory();
		IUndoContext undoContext = UiUtil.getWorkbenchUndoContext();
		operation.addContext(undoContext);
		try {
			operationHistory.execute(operation, null, null);
			operationHistory.add(operation);
		} catch (ExecutionException e) {
			e.printStackTrace();
		}
	}

	class ChangeHomotypicGroupOperation extends AbstractOperation {

		public ChangeHomotypicGroupOperation() {
			super("'" + text + "'");
		}

		@Override
		public IStatus execute(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			Action changeHomotypicGroupAction = null;
			if (taxonBase instanceof Synonym) {
				changeHomotypicGroupAction = new ChangeSynonymHomotypicGroupAction(
						(Synonym) taxonBase, taxon, newHomotypicGroup);
			}
			if (taxonBase instanceof Taxon) {
				changeHomotypicGroupAction = new ChangeTaxonRelationToSynonymAction(
						(Taxon) taxonBase, taxon, newHomotypicGroup);
			}
			changeHomotypicGroupAction
					.addPropertyChangeListener(new IPropertyChangeListener() {
						public void propertyChange(
								org.eclipse.jface.util.PropertyChangeEvent event) {
							if (event.getProperty().equals(
									ITaxEditorConstants.HOMOTYPIC_GROUP)) {

								Synonym newSynonym = null;
								if (event.getNewValue() instanceof Synonym) {
									newSynonym = (Synonym) event.getNewValue();
								} else {
									throw new IllegalArgumentException(
											"ChangeSynonymHomotypicGroupAction event did not return a Synonym in its newValue field.");
								}

								composite.setData(newSynonym);
								
								if (newSynonym.getName() == null) {
									return;
								}

								if (CdmUtil.isNameHomotypic(newSynonym.getName(), taxon)) {
									composite.transform(NameComposite.HOMOTYPIC_SYNONYM);
								} else {
									composite.transform(NameComposite.HETEROTYPIC_SYNONYM);
								}
							}
						}
					});
			changeHomotypicGroupAction.run();
			return Status.OK_STATUS;
		}

		@Override
		public IStatus redo(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			return execute(monitor, info);
		}

		@Override
		public IStatus undo(IProgressMonitor monitor, IAdaptable info)
				throws ExecutionException {
			return Status.OK_STATUS;
		}
	}
}