/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.datasource;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;

import eu.etaxonomy.cdm.api.application.CdmApplicationController;
import eu.etaxonomy.cdm.database.CdmDataSource;
import eu.etaxonomy.cdm.database.CdmPersistentDataSource;
import eu.etaxonomy.cdm.database.DatabaseTypeEnum;
import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.cdm.model.common.init.TermNotFoundException;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.model.CdmSessionDataRepository;
import eu.etaxonomy.taxeditor.model.CdmUtil;

public class CdmDataSourceRepository implements ICdmDataSourceRepository {
	private static final Logger logger = Logger
			.getLogger(CdmDataSourceRepository.class);
	
	private static ICdmDataSource currentDataSource;
	private static ICdmDataSourceRepository repository;
	private List<ICdmDataSource> dataSources;
	private CdmApplicationController cdmAppController;
	
	public static ICdmDataSourceRepository getDefault() {
		if (repository == null) {
			repository = new CdmDataSourceRepository();
			
			// Create datasource for local H2 database
			ICdmDataSource h2DataSource = CdmDataSource.NewH2EmbeddedInstance(
					"cdm", "sa", "");
			repository.add(h2DataSource);
						
			// Add stored datasources to repository
			for (ICdmDataSource dataSource : CdmPersistentDataSource.getAllDataSources()) {
				repository.add(dataSource);
			}
			
			// Get default datasource
			ICdmDataSource defaultDataSource = CdmPersistentDataSource.NewDefaultInstance();
			
			// If none specified, use local H2 database 
			if (defaultDataSource == null) {
				defaultDataSource = h2DataSource;
			}
			
			// Set repository's current datasource
			repository.setCurrentDataSource(defaultDataSource);
			
//			defaultDataSource = CdmPersistentDataSource.save("mysql_cichorieae", DatabaseTypeEnum.MySQL, "87.106.88.177", "cdm_edit_cichorieae", 80, "edit", "R3m0teAt80");
//			CdmDataSource mySqlDataSource = CdmDataSource.NewMySqlInstance(
//					"87.106.88.177", "cdm_edit_cichorieae", 80, "edit", "R3m0teAt80");
		}
		return repository;
	}

	@Override
	public boolean add(ICdmDataSource dataSource) {
		return getDataSources().add(dataSource);
	}


	@Override
	public boolean delete(CdmPersistentDataSource dataSource) {
		CdmPersistentDataSource.delete(dataSource);
		return remove(dataSource);
	}

	@Override
	public boolean remove(ICdmDataSource dataSource) {
		return getDataSources().remove(dataSource);
	}
	
	@Override
	public List<ICdmDataSource> getAll() {
		return getDataSources();
	}
		
	@Override
	public ICdmDataSource getCurrentDataSource() {
		if (currentDataSource == null) {
			throw new IllegalStateException("Current data source not set.");
		}
		return currentDataSource;
	}

	@Override
	public boolean setCurrentDataSource(ICdmDataSource dataSource) {
		if (currentDataSource != null) {
			if (!changeDataSource(dataSource)) {
				return false;
			}
		}
		currentDataSource = dataSource;
		return true;
	}

	@Override
	public void setCdmApplicationController(
			CdmApplicationController cdmAppController) {
		this.cdmAppController = cdmAppController;		
	}
	
	private CdmApplicationController getCdmAppController() {
		if (cdmAppController == null) {
			throw new IllegalStateException("CdmApplicationController not set.");
		}
		return cdmAppController;
	}
	
	private List<ICdmDataSource> getDataSources() {
		if (dataSources == null) {
			dataSources = new ArrayList<ICdmDataSource>();
		}
		return dataSources;
	}
		
	private boolean changeDataSource(final ICdmDataSource dataSource) {

		// Close all open editors before showing progress monitor
		if (!UiUtil.closeOpenTaxonEditors()) {
			
			// User has canceled operation   
			return false;
		}
		
		try {
			IRunnableWithProgress op = new IRunnableWithProgress() {

				@Override
				public void run(IProgressMonitor monitor)
						throws InvocationTargetException, InterruptedException {
				
					// Start the progress bar with 25 steps
					monitor.beginTask("Changing to a new datasource ...", 25);
				
					// Clear all session variables
					CdmSessionDataRepository.getDefault().clearNonTaxonData();
					monitor.worked(1);
					
					// Clear session taxa
					CdmSessionDataRepository.getDefault().clearTaxonData();
					monitor.worked(1);
					
					CdmTransactionController.commitTransaction();
					monitor.worked(1);
										
					// Set new data source
					try {
						getCdmAppController().changeDataSource(dataSource);
					} catch (TermNotFoundException e) {
						e.printStackTrace();
						throw new InterruptedException();
					}
					monitor.worked(10);
					
					CdmTransactionController.startTransaction();
					monitor.worked(1);
										
					// Get session root taxa
					// TODO get listener in tree content provider to take care of this
					UiUtil.getTreeViewer().setInput(
							CdmSessionDataRepository.getDefault().getRootTaxa());
							
					monitor.worked(11);
										
					monitor.done();
				}
				
			};
			new ProgressMonitorDialog(UiUtil.getShell()).run(false, true, op);
		} catch (InvocationTargetException e) {
			e.printStackTrace();
			return false;
		} catch (InterruptedException e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

	@Override
	public CdmPersistentDataSource save(String dataSourceName, DatabaseTypeEnum databaseType,
			String server, String database, int port, String username,
			String password) {
		CdmPersistentDataSource dataSource = CdmPersistentDataSource.save(
				dataSourceName, databaseType, server, database, port, username, password);
		add(dataSource);
		return dataSource;
	}
	
	@Override
	public CdmPersistentDataSource save(String dataSourceName, DatabaseTypeEnum databaseType,
			String server, String database, String username, String password) {
		CdmPersistentDataSource dataSource = CdmPersistentDataSource.save(
				dataSourceName, databaseType, server, database, username, password);
		add(dataSource);
		return dataSource;
	}
}