/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;

import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;

/**
 * Draws a border on a Composite on an IManagedForm when it gets focus, 
 *  undraws it when the Composite loses focus.
 *  
 * @author p.ciardelli
 *
 */
public class CompositeBorderDecorator {
	private FormToolkit toolkit;
	private ScrolledForm scrolledForm;
	private Composite borderedComposite;
	
	private FocusListener listener = new PaintBorderListener();
	
	/**
	 * If true, the <code>borderedComposite</code>'s border is erased when
	 * it loses focus.
	 */
	private boolean doLoseFocus = true;

	public CompositeBorderDecorator(Composite composite, IManagedForm form) {
		this.borderedComposite = composite;
		this.toolkit = form.getToolkit();
		this.scrolledForm = form.getForm();
		
		toolkit.adapt(composite);
		
		Color groupBackgroundColor = TaxEditorPlugin.getDefault().
				getColor(ITaxEditorConstants.GROUP_GRAY_BKG_COLOR); 
		composite.setBackground(groupBackgroundColor);
		
		composite.addFocusListener(listener);
	}
	
	/**
	 * If this is set to true, border will disappear when focus is lost.
	 * Otherwise, it will remain until explicitly erased. 
	 * 
	 * @param doLoseFocus
	 */
	public void setLoseFocus(boolean doLoseFocus) {
		this.doLoseFocus = doLoseFocus;
	}
	
	/**
	 * Removes the border from <code>borderedComposite</code>.
	 */
	public void unpaintBorder() {
		paintBorder(null);		
	}
	
	/**
	 * Adds a border to <code>borderedComposite</code>.
	 */
	public void paintBorder() {
		paintBorder(FormToolkit.TEXT_BORDER);
	}
	
	private void paintBorder(String border) {		
		borderedComposite.setData(FormToolkit.KEY_DRAW_BORDER, border);
		toolkit.paintBordersFor(borderedComposite.getParent());
		scrolledForm.reflow(false);
	}
	
	/**
	 * If the border should actually be drawn elsewhere when 
	 *  the main Composite gets focus, i.e. the textviewer should
	 *  get a border when its Composite has focus, set the
	 *  Composite here.
	 * 
	 * @param composite
	 */
	public void setBorderedComposite(Composite composite) {
		this.borderedComposite = composite;
		borderedComposite.addFocusListener(listener);
	}
	
	class PaintBorderListener implements FocusListener {
		public void focusGained(FocusEvent e) {
			paintBorder();
		}

		public void focusLost(FocusEvent e) {
			if (doLoseFocus) {
				unpaintBorder();
			}
		}
	}
}