/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.text.TextViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import eu.etaxonomy.cdm.model.taxon.Taxon;

/** 
 * Formats <code>GroupedComposite</code> with cosmetic and layout properties specific to the
 * Editor. This should be used to maintain a consistent look and feel for all Editor 
 * freetext area components, such as DescriptionElementComposite.
 * <p>
 * Requires an <code>IManagedForm</code>, whose <code>input</code> is set to the contents
 * of {@link #getData()} when the <code>EditorGroupedComposite</code> gets focus, i.e. to
 * populate the property sheet with the data.
 * </p>
 * <p>
 * The <code>IManagedForm</code> is also required to have a <code>Taxon</code> in its 
 * own <code>getData()</code>.
 * </p> 
 * <p>
 * The <code>IManagedForm</code> can also used for drawing borders by calling the method 
 * <code>createBorderSupport()</code>.
 * </p>
 * @author p.ciardelli
 * @created 02.06.2008
 * @version 1.0
 */
abstract public class EditorGroupedComposite extends GroupedComposite implements IHasPropertySource {
	private static final Logger logger = Logger.getLogger(EditorGroupedComposite.class);
	
	protected TextViewer textViewer;
	protected TableWrapLayout layout;
	private Label draggableLabel;
	protected IManagedForm managedForm;
	protected Taxon taxon;

	private CompositeBorderDecorator borderDecorator;
	
	/**
	 * @param parent
	 * @param managedForm
	 */
	public EditorGroupedComposite(Composite parent, IManagedForm managedForm) {
		super(parent);
		
		this.managedForm = managedForm;
		
		Object formData = managedForm.getForm().getBody().getData();
		Assert.isTrue(formData instanceof Taxon, 
				"Managed form must have a Taxon in its data field.");
		taxon = (Taxon) formData;
	}

	/**
	 * 
	 */
	protected void createLineWrapSupport() {
		new LineWrapSupport(getTextViewer(), managedForm);
	}
		
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.GroupedComposite#createContent()
	 */
	@Override
	protected void createContent() {
		setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB));
		layout = new TableWrapLayout();
		layout.numColumns = 2;
		layout.leftMargin = 0;
		setLayout(layout);
		
		setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		
		draggableLabel = new Label(this, SWT.NONE);
		draggableLabel.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		draggableLabel.setLayoutData(new TableWrapData(TableWrapData.CENTER, TableWrapData.TOP));
		
		setDraggableControl(draggableLabel);
	}
	
	public Taxon getTaxon() {
		return taxon;
	}
	
	/**
	 * @return
	 */
	public TextViewer getTextViewer() {
		return textViewer;
	}

	/**
	 * @param textViewer
	 */
	public void setTextViewer(TextViewer textViewer) {
		this.textViewer = textViewer;
	}

	public void setText(String text) {
		Assert.isNotNull(getTextViewer(), 
				"Cannot set text for a TextViewer that has not yet been initialized.");
		Assert.isNotNull(getTextViewer().getDocument(), 
				"Cannot set text for a TextViewer whose Document has not yet been initialized.");
		getTextViewer().getDocument().set(text);
	}
	
	/**
	 * @param icon
	 */
	public void setIcon(Image icon) {
		draggableLabel.setImage(icon);
	}

	/**
	 * @return
	 */
	public Label getDraggableLabel() {
		return draggableLabel;
	}
	
	/**
	 * @param indent
	 */
	public void setIndent(int indent) {
		layout.leftMargin = indent;
		this.setLayout(layout);
	}	
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.widgets.Composite#setFocus()
	 */
	public boolean setFocus() {
		setSelection();
		return super.setFocus();
	}
	
	/**
	 * 
	 */
	protected void setSelection() {
		managedForm.setInput(this);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.widgets.Control#setFont(org.eclipse.swt.graphics.Font)
	 */
	public void setFont(Font font) {
		getTextViewer().getTextWidget().setFont(font);
	}

	/**
	 * If <code>textViewer</code> has already been set, it will show a 
	 * <code>prompt</code> along the lines of "Click here to start entering data" 
	 * when empty.
	 * 
	 * @param prompt
	 */
	/**
	 * @param prompt
	 */
	public void createEmptyViewerPrompt(String prompt) {
		
		Assert.isNotNull(getTextViewer());
		
		new EmptyTextViewerPrompt(getTextViewer().getDocument(), 
				getTextViewer().getTextWidget(), prompt);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.widgets.Control#getMenu()
	 */
	public Menu getMenu () {
		
		Menu menu = draggableLabel.getMenu();
		if (menu == null) {
			menu = new Menu(draggableLabel);
			draggableLabel.setMenu(menu);
		}
		return menu;
	}
	
	/**
	 * 
	 */
	public void createBorderSupport() {
		borderDecorator = new CompositeBorderDecorator(
				this, managedForm);
		if (getTextViewer() == null) {
			logger.warn("Could not create border support - getTextViewer() returned null.");
		} else {
			borderDecorator.setBorderedComposite(getTextViewer().getTextWidget());
			borderDecorator.setLoseFocus(false);
		}
	}
	
	public void drawBorder() {
		borderDecorator.paintBorder();
	}

	/**
	 * @param isDirty
	 */
	protected void setDirty(boolean isDirty) {
		managedForm.dirtyStateChanged();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.GroupedComposite#dispose()
	 */
	public void dispose () {
		
		// Nullify decorator to stop its listener to listening for
		//	composite focus
		borderDecorator = null;
		
		super.dispose();
		
		// A disposed graphic element means that the taxon has changed
		setDirty(true);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.GroupedComposite#setParent(org.eclipse.swt.widgets.Composite)
	 */
	public boolean setParent(Composite parent) {
		if (super.setParent(parent)) {
			
			// Has this been moved to a new IManagedForm?
			if (parent instanceof EditorGroupComposite) {
				IManagedForm parentForm = ((EditorGroupComposite) parent)
						.getManagedForm();
				if (!parentForm.equals(managedForm)) {
					managedForm = parentForm;
				}
			}
			
			setDirty(true);
			return true;
		}
		return false;
	}
	
	
	/**
	 * Returns a <code>ContextMenu</code> opened by right-clicking on
	 * <code>draggableLabel</code>.
	 *  
	 * @return
	 */
	protected ContextMenu createContextMenu() {
		return new ContextMenu(getDraggableLabel());
	}
	
	public void unpaintBorder() {
		if (borderDecorator != null) {
			borderDecorator.unpaintBorder();
		}
	}
} 