/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.description;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.AbstractTaxonEditor;
import eu.etaxonomy.taxeditor.editor.EditorGroupComposite;
import eu.etaxonomy.taxeditor.editor.EditorGroupedComposite;
import eu.etaxonomy.taxeditor.editor.IHasPropertySource;

/**
 * Displays the <code>TaxonDescription</code>s attached to a <code>Taxon</code>.
 * <p>
 * For each <code>TaxonDescription</code>:
 * <ul>
 * 	<li>Displays a <code>DescriptionLabelComposite</code> with <code>TaxonDescription#getTextCache()</code></li>
 * 	<li>Displays a <code>FeatureGroupComposite</code> for each <code>Feature</code> for which there are descriptive elements</li>
 * 	<li>Displays a <code>DescriptionElementComposite</code> for each <code>TaxonDescription#getElements()</code></li>
 * </ul>
 * </p> 
 * <p>
 * If the <Taxon> has no descriptions, creates a <code>TaxonDescription</code> that is saved when 
 * <code>firePropertyChange()</code> is called with <code>PROP_DIRTY</code>.
 * </p>
 * @author p.ciardelli
 * @created 25.08.2008
 * @version 1.0
 */
public class TaxonDescriptionEditorView extends AbstractTaxonEditor {
	private static final Logger logger = Logger
			.getLogger(TaxonDescriptionEditorView.class);
	
	/**
	 * If this view is opened and the <code>Taxon</code> has no <code>Description</code>,
	 * a <code>Description</code> element is created and saved to <code>emptyDescription</code>.
	 * It should not be added to the <code>Taxon</code> until it has been changed.
	 * 
	 * @see firePropertyChange(int)
	 */
	private TaxonDescription emptyDescription;
	
	private boolean pageInitialized = false; 

	@Override
	public void doSave(IProgressMonitor monitor) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void doSaveAs() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public boolean isDirty() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean isSaveAsAllowed() {
		// TODO Auto-generated method stub
		return false;
	}
	
	@Override
	public void createPartControl(Composite composite) {

		super.createPartControl(composite);
		
		Taxon taxon = getTaxon();
		
		Set<TaxonDescription> descriptions = taxon.getDescriptions();
		
		EditorGroupedComposite firstEditableComposite = null;
		
		// If the Taxon has no descriptions, give it an empty one to display
		if (descriptions == null || descriptions.size() == 0) {
			emptyDescription = TaxonDescription.NewInstance();
			TaxonDescription description = emptyDescription;
			description.setTitleCache("");
			descriptions.add(description);
		}
		
		// Iterate through taxon's descriptions
		for (TaxonDescription description : descriptions) {
						
			// Create a composite for the description
			EditorGroupComposite descriptionComposite = new EditorGroupComposite(
					parent, managedForm, description);
			descriptionComposite.setDroppable(false);
			
			// Add a grouped composite for the description's label
			EditorGroupedComposite descriptionLabelComposite = new DescriptionLabelComposite(
					descriptionComposite, managedForm, description);
			
			if (firstEditableComposite == null) {
				firstEditableComposite = descriptionLabelComposite;
			}
			
			// Iterate through description's elements, group by feature
			Map<Feature, Set<DescriptionElementBase>> features =  
					new HashMap<Feature, Set<DescriptionElementBase>>(); 
			for (DescriptionElementBase element : description.getElements()) {
				Feature feature = element.getFeature();
				Set<DescriptionElementBase> elements = features.get(feature);
				if (elements == null) {
					elements = new HashSet<DescriptionElementBase>();
					features.put(feature, elements);
				}
				elements.add(element);
			}
			
			// Create a composite for each feature, composites for its elements
			for (Feature feature : features.keySet()) {
				EditorGroupComposite featureComposite = new FeatureGroupComposite(
						descriptionComposite, managedForm, feature);
				
				for (DescriptionElementBase element : features.get(feature)) {
					new DescriptionElementComposite(
							featureComposite, managedForm, element);
				}
			}
		}
		
		// We've added elements to the managed form, so redraw it
		scrolledForm.reflow(true);
		
		// Set focus on the first label composite to give a border and send it
		//	to the property sheet
		setSelection(firstEditableComposite);
		firstEditableComposite.drawBorder();
		
		pageInitialized = true;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#firePropertyChange(int)
	 */
	protected void firePropertyChange(final int propertyId) {
		if (pageInitialized && propertyId == PROP_DIRTY && emptyDescription != null) {
			getTaxon().addDescription(emptyDescription);
			emptyDescription = null;
		}
		super.firePropertyChange(propertyId);
	}
}
