/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.name;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.action.Action;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.views.properties.IPropertySource;

import eu.etaxonomy.cdm.model.name.BotanicalName;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.name.ZoologicalName;
import eu.etaxonomy.cdm.model.reference.StrictReferenceBase;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.actions.ui.CreateNewHeterotypicCompositeAction;
import eu.etaxonomy.taxeditor.editor.EditorGroupedComposite;
import eu.etaxonomy.taxeditor.editor.LineBreakListener;
import eu.etaxonomy.taxeditor.editor.ParseListener;
import eu.etaxonomy.taxeditor.model.CdmUtil;
import eu.etaxonomy.taxeditor.operations.name.ChangeHomotypicGroupOperation;
import eu.etaxonomy.taxeditor.operations.name.CreateSynonymInNewGroupOperation;
import eu.etaxonomy.taxeditor.propertysheet.name.BotanicalNamePropertySource;
import eu.etaxonomy.taxeditor.propertysheet.name.NonViralNamePropertySource;
import eu.etaxonomy.taxeditor.propertysheet.name.TaxonBasePropertySource;
import eu.etaxonomy.taxeditor.propertysheet.name.ZoologicalNamePropertySource;

/**
 * Formats an <code>EditorGroupedComposite</code> to display <code>TaxonNameBase</code> elements
 * in a <code>NameViewer</code>.
 * 
 * <code>Composite.getData()</code> returns <code>TaxonBase</code>. Therefore, the method   
 * <code>setSelection()</code> is overriden to send the property sheet <code>TaxonBase.getName()</code>. 
 * 
 * @author p.ciardelli
 * @created 02.06.2008
 * @version 1.0
 */
public class NameComposite extends EditorGroupedComposite {
	private static final Logger logger = Logger.getLogger(NameComposite.class);

	/**
	 * ************ COMPOSITE TYPES ************
	 */
	public String compositeType;
	public static final String ACCEPTED_TAXON = "accepted_name_composite";
	public static final String HOMOTYPIC_SYNONYM = "homotypic_name_composite";
	public static final String HETEROTYPIC_SYNONYM = "heterotypic_name_composite";
	public static final String MISAPPLIED_NAME = "misappliedname_name_composite";

	/**
	 * ************ INDENTATIONS ************
	 */
	public static final int ACCEPTED_INDENT = 0;
	public static final int SYNONYM_INDENT = 15;
	public static final int MISAPPLIEDNAME_INDENT = 15;

	/**
	 * ************ FONTS ************
	 */
	public static final Font ACCEPTED_FONT = TaxEditorPlugin.getDefault()
			.getFont(ITaxEditorConstants.ACCEPTED_TAXON_FONT);
	public static final Font SYNONYM_FONT = TaxEditorPlugin.getDefault()
			.getFont(ITaxEditorConstants.SYNONYM_FONT);
	public static final Font MISAPPLIEDNAME_FONT = TaxEditorPlugin.getDefault()
			.getFont(ITaxEditorConstants.MISAPPLIEDNAME_FONT);

	/**
	 * ************ ICONS ************
	 */
	public static final Image ACCEPTED_ICON = TaxEditorPlugin.getDefault()
			.getImage(ITaxEditorConstants.BLACK_SQUARE_ICON);
	public static final Image HOMOTYPIC_SYNONYM_ICON = TaxEditorPlugin
			.getDefault().getImage(ITaxEditorConstants.HOMOTYPIC_SYN_ICON);
	public static final Image HOMOTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON = TaxEditorPlugin
			.getDefault().getImage(
					ITaxEditorConstants.HOMOTYPIC_SYN_ORIGINAL_ICON);
	public static final Image HETEROTYPIC_SYNONYM_ICON = TaxEditorPlugin
			.getDefault().getImage(ITaxEditorConstants.HETEROTYPIC_SYN_ICON);
	public static final Image HETEROTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON = TaxEditorPlugin
			.getDefault().getImage(
					ITaxEditorConstants.HETEROTYPIC_SYN_ORIGINAL_ICON);
	public static final Image MISAPPLIEDNAME_ICON = TaxEditorPlugin
			.getDefault().getImage(ITaxEditorConstants.MISAPPLIED_NAME_ICON);
	public static final Image AUTONYM_ICON = TaxEditorPlugin.getDefault()
			.getImage(ITaxEditorConstants.AUTONYM_ICON);
	public static final Image BASIONYM_ICON = TaxEditorPlugin.getDefault()
			.getImage(ITaxEditorConstants.BASIONYM_ICON);
	public static final Image MOVE = TaxEditorPlugin.getDefault().getImage(
			ITaxEditorConstants.MOVE_ICON);

	/**
	 * ************ TRANSFORMATIONS ************
	 */
	public static final String ADD_GROUP_BASIONYM = "add_group_basionym";
	public static final String REMOVE_GROUP_BASIONYM = "remove_group_basionym";

	/**
	 * ************ MENU ACTIONS ************
	 */
	public Action CHANGE_TAXON_TO_SYNONYM_ACTION;

	private static final String EMPTY_NAME_PROMPT = "Click to add name";

	/**
	 * Used to turn parser on and off.
	 * 
	 * @see activateParser
	 * @see deactivateParser
	 */
	private boolean isUseParser = false;

	protected boolean isParsing;

	private NameViewer nameViewer;

	/**
	 * The constructor for a DescriptionElementComposite. Takes a parent Composite on which to
	 * create itself, and an IManagedForm for Composite life cycle methods, i.e.
	 * drawing borders, creating other Composites, creating line wrap support,
	 * etc.
	 * 
	 * @param parent
	 * @param managedForm
	 */
	public NameComposite(Composite parent, IManagedForm managedForm, 
			String compositeType, TaxonBase data) {
		super(parent, managedForm);

		createNameViewer();
		createBorderSupport();
		createLineWrapSupport();
		
		setData(data);
//		if (compositeType != null) {
//			transform(compositeType);
//		}

		createParser();	
		createEmptyViewerPrompt(EMPTY_NAME_PROMPT);

		setFocus();
		
		// NOTE: placing this after setFocus() solved a strange bug where if the first action
		//	during a session was to open a new taxon, the empty name prompt didn't work
		createNameListener(data);
	}

	/**
	 * Listens for changes to this name's <code>fullTitleCache</code>.
	 * 
	 * @param data
	 */
	private void createNameListener(TaxonBase taxonBase) {
		if (taxonBase.getName() == null) {
			return;
		}
		
		final TaxonNameBase name = taxonBase.getName();
		
		final PropertyChangeListener listener = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				
				if (isParsing) {
					return;
				}
				
				if (UiUtil.getIsSaving()) {
					return;
				}
				
				deactivateParser();
				((NameViewer) getTextViewer()).setText(name.getFullTitleCache());
				activateParser();
			}
		};
		
		// TODO clean this part up
		name.addPropertyChangeListener("fullTitleCache", listener);
		name.addPropertyChangeListener("nomenclaturalMicroReference", listener);

//		name.addPropertyChangeListener(ITaxEditorConstants.REFRESH_NAMEVIEWER, listener);
		
		StrictReferenceBase reference = (StrictReferenceBase) name.getNomenclaturalReference();
		if (reference != null) {
			reference.addPropertyChangeListener("titleCache", listener);
		}
	}

	/**
	 * Override of Composite.setData() which passes data along to
	 * <code>NameComposite</code>'s <code>TextViewer</code> where appropriate.
	 * 
	 * @see org.eclipse.swt.widgets.Widget#setData(java.lang.Object)
	 */
	public void setData(Object data) {
		super.setData(data);
		
		Assert.isTrue((data instanceof TaxonBase), 
				"NameComposite's data field must contain a TaxonBase object");
		
		String text = CdmUtil.getDisplayNameAndRef((TaxonBase) data);
		getTextViewer().getTextWidget().setText(text);

		if (getTextViewer() instanceof NameViewer) {
			((NameViewer) getTextViewer()).setCursorToEOL();

			TaxonNameBase name = ((TaxonBase) data).getName();
			if (name != null) {
				
				boolean hasProblem = name.getHasProblem();
				((NameViewer) getTextViewer()).setShowError(name);
			}
		}
		
	}

	private void createParser() {
		((NameViewer) getTextViewer()).setParseListener(new ParseListener() {

			@Override
			public void parse(String text) {
				
				// Either composite is not yet fully built, or
				//	the property sheet is writing to it
				if (!isUseParser) {
					return;
				}
				
				// Let others know the parser is active
				isParsing = true;
				
				// Get the name from the composite's data field
				TaxonBase taxonBase = (TaxonBase) getData();
				NonViralName nonViralName = (NonViralName) taxonBase.getName();
				
				// Parse the name and paint the text field w any errors
				if (nonViralName != null) {
					CdmParserController.parseFullReference(nonViralName, text);
					((NameViewer) getTextViewer()).setShowError(nonViralName);
				}

				// Any entry of text means the taxon has been changed
				setDirty(true);
				
				// The parser is no longer active
				isParsing = false;

				// Manually refresh the property sheet to reflect changes
				UiUtil.getPropertySheetPage().refresh();
			}
		});
		activateParser();
	}

	public void activateParser() {
		isUseParser = true;
	}
	
	public void deactivateParser() {
		isUseParser = false;
	}
	
	private NameViewer createNameViewer() {
//		nameViewer = new NameViewer(this);
//		nameViewer.setLineBreakListener(new LineBreakListener() {
		
		setTextViewer(new NameViewer(this));
		getTextViewer().setLineBreakListener(new LineBreakListener() {

			@Override
			public void handleSplitText(String text) {

				Composite parent = getParent();
				final Composite grandParent = parent.getParent();

//				new CreateNewHeterotypicCompositeAction(text, managedForm)
//						.run();
				
				IOperationHistory operationHistory = UiUtil.getOperationHistory();
				IUndoContext undoContext = UiUtil.getTaxonNameEditorUndoContext(taxon);
				IUndoableOperation operation = new CreateSynonymInNewGroupOperation
						("new heterotypic synonym", undoContext, taxon, text); //$NON-NLS-1$
				
				try {
					IStatus status = operationHistory.execute(operation, null, null);
				} catch (ExecutionException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

				// setDirty(true);
			}

		});

		getTextViewer().getTextWidget().addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				setFocus();
			}
		});

		// createLineWrapSupport(nameViewer);

		getTextViewer().getTextWidget().setBackground(TaxEditorPlugin.getDefault().
				getColor(ITaxEditorConstants.GROUP_GRAY_BKG_COLOR));		
		
		return (NameViewer) getTextViewer();
	}
	
	public void setText(String text) {
		Assert.isNotNull(getTextViewer(), 
				"Cannot set text for a TextViewer that has not yet been initialized.");
		Assert.isNotNull(getTextViewer().getDocument(), 
				"Cannot set text for a TextViewer whose Document has not yet been initialized.");
		getTextViewer().getDocument().set(text);
	}
	
	public NameViewer getTextViewer() {
		return this.nameViewer;
	}
	
	public void setTextViewer(NameViewer textViewer) {
		this.nameViewer = textViewer;
	}

	public IPropertySource getPropertySource() {
		
		// Property source for misapplied name
		if (MISAPPLIED_NAME.equals(compositeType)) {
			return new TaxonBasePropertySource((TaxonBase) getData());
		}
		
		// Get name info out of composite data
		TaxonBase taxon = (TaxonBase) getData();
		if (taxon == null) {
			return null;
		}
		TaxonNameBase name = taxon.getName();
		if (name == null) {
			return null;
		}
		
		// Property sources for names
		if (name.getClass() == BotanicalName.class) {
			return new BotanicalNamePropertySource((BotanicalName) name);
		}
		if (name.getClass() == ZoologicalName.class) {
			return new ZoologicalNamePropertySource((ZoologicalName) name);
		}
		if (name instanceof NonViralName) {
			return new NonViralNamePropertySource((NonViralName) name);
		}
		
		return null;
	}
}