/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.IManagedForm;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.editor.ContextMenu;
import eu.etaxonomy.taxeditor.model.CdmUtil;
import eu.etaxonomy.taxeditor.operations.name.ChangeHomotypicGroupOperation;
import eu.etaxonomy.taxeditor.operations.name.ChangeSynonymToMisappliedNameOperation;
import eu.etaxonomy.taxeditor.operations.name.ChangeSynonymToTaxonOperation;
import eu.etaxonomy.taxeditor.operations.name.MakeSynonymAcceptedOperation;
import eu.etaxonomy.taxeditor.operations.name.MakeSynonymGroupBasionymOperation;
import eu.etaxonomy.taxeditor.operations.name.RemoveSynonymOperation;
import eu.etaxonomy.taxeditor.operations.name.UnmakeSynonymGroupBasionymOperation;

/**
 * @author p.ciardelli
 * @created 13.01.2009
 * @version 1.0
 */
public class SynonymComposite extends NameComposite {
	private static final Logger logger = Logger
			.getLogger(SynonymComposite.class);
	
	private Synonym synonym;
	
	public SynonymComposite(Composite parent, IManagedForm form,
			Taxon taxon, Synonym synonym) {
		super(parent, form, null, synonym);
		
		this.taxon = taxon;
		this.synonym = synonym;
		
		setDraggable(true);
		setFont(SYNONYM_FONT);
		setIndent(SYNONYM_INDENT);
		
		if (isHomotypic()) {
			if (CdmUtil.isNameGroupBasionym(synonym.getName())) {
				setIcon(HOMOTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON);
			} else {
				setIcon(HOMOTYPIC_SYNONYM_ICON);
			}	
		} else {
			if (CdmUtil.isNameGroupBasionym(synonym.getName())) {
				setIcon(HETEROTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON);
			} else {
				setIcon(HETEROTYPIC_SYNONYM_ICON);
			}
		}

		createMenu();
	}
	
	private boolean isHomotypic() {
		HomotypicalGroup group = synonym.getHomotypicGroup();
		return group.equals(taxon.getHomotypicGroup()); 
	}
	
	public Synonym getSynonym() {
		return synonym;
	}
	
	public boolean setParent(Composite parent) {

		if (super.setParent(parent)) {	
			
			// Has this been moved to the misapplied names group?
			if (parent instanceof MisappliedGroupComposite) {
				
				IOperationHistory operationHistory = UiUtil.getOperationHistory();
				IUndoContext undoContext = UiUtil.getTaxonNameEditorUndoContext(taxon);
				IUndoableOperation operation = new ChangeSynonymToMisappliedNameOperation
						("change to misapplication", undoContext, taxon, synonym); //$NON-NLS-1$
				
				try {
					IStatus status = operationHistory.execute(operation, null, null);
				} catch (ExecutionException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}

			// Has this been moved to a HomotypicalGroup?
			if (parent instanceof HomotypicalGroupComposite) {
				
				HomotypicalGroup homotypicalGroup = 
						((HomotypicalGroupComposite)parent).getGroup();
				
				// Make sure we are not dropping synonym on its own group
				if (homotypicalGroup.equals(synonym.getHomotypicGroup())) {
					return true;
				}

				IOperationHistory operationHistory = UiUtil.getOperationHistory();
				IUndoContext undoContext = UiUtil.getTaxonNameEditorUndoContext(taxon);
				IUndoableOperation operation = new ChangeHomotypicGroupOperation
						("change type", undoContext, taxon, synonym, homotypicalGroup); //$NON-NLS-1$
				
				try {
					IStatus status = operationHistory.execute(operation, null, null);
				} catch (ExecutionException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			return true;

		}
		return false;
	}
	
	/**
	 * @param nameComposite
	 */
	private void createMenu() {

		ContextMenu contextMenu = createContextMenu();

		final IOperationHistory operationHistory = UiUtil.getOperationHistory();
		final IUndoContext undoContext = UiUtil.getTaxonNameEditorUndoContext(taxon);
		
		// Change composite to a misapplied name
		String text = "Change synonym to misapplied name"; //$NON-NLS-1$
		ImageDescriptor image = TaxEditorPlugin.getDefault()
				.getImageDescriptor(ITaxEditorConstants.MISAPPLIED_NAME_ICON);
		contextMenu.addAction(new Action(text, image){
						
			public void run() {
				IUndoableOperation operation = new ChangeSynonymToMisappliedNameOperation
						(this.getText(), undoContext, taxon, synonym);
				try {
					IStatus status = operationHistory.execute(operation, null, null);
				} catch (ExecutionException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		});
		
		// Remove composite
		text = "Remove synonym from taxon"; //$NON-NLS-1$
		image = TaxEditorPlugin.getDefault()
				.getImageDescriptor(ITaxEditorConstants.ACTIVE_DELETE_ICON);
		contextMenu.addAction(new Action(text, image){
			
			public void run() {
				IUndoableOperation operation = new RemoveSynonymOperation
						(this.getText(), undoContext, taxon, synonym);
				try {
					IStatus status = operationHistory.execute(operation, null, null);
				} catch (ExecutionException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}				
			}
		});
		
		// Separator
		contextMenu.addSeparator();
				
		// Make the synonym the basionym of the homotypic group
		text = Messages.getString
				("AddBasionymCompositeAction.0", synonym.getName());  //$NON-NLS-1$
		image = TaxEditorPlugin.getDefault()
					.getImageDescriptor(ITaxEditorConstants.BASIONYM_ICON);
		Action makeBasionymAction = new Action(text, image){
			
			public void run() {
				IUndoableOperation operation = new MakeSynonymGroupBasionymOperation
						(this.getText(), undoContext, taxon, synonym);
				try {
					IStatus status = operationHistory.execute(operation, null, null);
				} catch (ExecutionException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}				
			}
		};
		contextMenu.addAction(makeBasionymAction);	
	
		// Action to remove the composite as basionym of the homotypic group
		text = Messages.getString
				("RemoveBasionymCompositeAction.0", synonym.getName());  //$NON-NLS-1$
		image = TaxEditorPlugin.getDefault()
					.getImageDescriptor(ITaxEditorConstants.BASIONYM_ICON);
		Action unmakeBasionymAction = new Action(text, image){
			
			public void run() {
				IUndoableOperation operation = new UnmakeSynonymGroupBasionymOperation
						(this.getText(), undoContext, taxon, synonym);
				try {
					IStatus status = operationHistory.execute(operation, null, null);
				} catch (ExecutionException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}				
			}
		};
		contextMenu.addAction(unmakeBasionymAction);
		
		if (CdmUtil.isNameGroupBasionym(synonym.getName())) {
			
			// Gray out action for "make synonym group basionym"
			makeBasionymAction.setEnabled(false);
		} else {
			
			// Gray out action for "unmake synonym group basionym"
			unmakeBasionymAction.setEnabled(false);
		}
		
		// Separator
		contextMenu.addSeparator();

		// Change this synonym to the taxon's accepted name
		text = "Make synonym this taxon's accepted taxon"; //$NON-NLS-1$
		image = TaxEditorPlugin.getDefault()
					.getImageDescriptor(ITaxEditorConstants.SWAP_SYNONYM_AND_TAXON_ICON);
		Action swapAction = new Action(text, image){
			
			public void run() {
				IUndoableOperation operation = new MakeSynonymAcceptedOperation
						(this.getText(), undoContext, taxon, synonym);
				try {
					IStatus status = operationHistory.execute(operation, null, null);
				} catch (ExecutionException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}				
			}
		};
		contextMenu.addAction(swapAction);
		// TODO fix operation
		swapAction.setEnabled(false);
		
		// Make a new taxon with this synonym as the accepted name
		text = "Use synonym name to make a new taxon"; //$NON-NLS-1$
		image = TaxEditorPlugin.getDefault()
					.getImageDescriptor(ITaxEditorConstants.SYNONYM_TO_TAXON_ICON);

		contextMenu.addAction(new Action(text, image){
			
			public void run() {
				IUndoableOperation operation = new ChangeSynonymToTaxonOperation
						(this.getText(), undoContext, taxon, synonym);
				try {
					IStatus status = operationHistory.execute(operation, null, null);
				} catch (ExecutionException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}				
			}
		});
	}
}