/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations.name;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PartInitException;

import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.actions.ui.OpenTaxonEditorAction;
import eu.etaxonomy.taxeditor.model.CdmSessionDataRepository;
import eu.etaxonomy.taxeditor.navigation.RecentNamesView;
import eu.etaxonomy.taxeditor.operations.AbstractEditorOperation;

/** 
 * @author p.ciardelli
 * @created 15.01.2009
 * @version 1.0
 */
public class MakeSynonymAcceptedOperation extends AbstractEditorOperation {
	private static final Logger logger = Logger
			.getLogger(MakeSynonymAcceptedOperation.class);
	private Synonym synonym;
	private Taxon parentTaxon;

	public MakeSynonymAcceptedOperation(String text, IUndoContext undoContext,
			Taxon taxon, Synonym synonym) {
		super(text, undoContext, taxon);
		
		this.synonym = synonym;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		// If taxon editor is dirty force user to save before proceeding, then close editor
		try {
			IEditorPart oldEditor = UiUtil.getEditorByTaxon(taxon);
			if (oldEditor.isDirty()) {
				if (!MessageDialog.openConfirm(UiUtil.getShell(), "Save before proceeding", 
						"All changes must be saved before proceeding with this action.\n\n" +
						"Press \"OK\" to save and continue, or \"Cancel\" to cancel this action.")) {
					return Status.CANCEL_STATUS;
				}
				oldEditor.doSave(null);
				UiUtil.closeEditor(oldEditor, true);
			}
		} catch (PartInitException e) {
			e.printStackTrace();
			MessageDialog.openError(UiUtil.getShell(), "Error", "Error saving and closing taxon");
			return Status.CANCEL_STATUS;
		}
		
		// Create new taxon for synonym to be swapped into
		TaxonNameBase synonymName = synonym.getName();
		Taxon newAcceptedTaxon = Taxon.NewInstance(synonymName, synonym.getSec());
		
		// Get taxon service and call its swap method
		ITaxonService taxonService = TaxEditorPlugin.getDefault().getApplicationController().getTaxonService();
		// TODO prompt user for citation 
		// TODO bug, anahit. new accepted taxon is not removed from synonymy 
		taxonService.makeTaxonSynonym(taxon, newAcceptedTaxon, null, null, null);
		
		// Remove old taxon from tree
		CdmSessionDataRepository.getDefault().removeTaxon(taxon);

		// Add new taxon to tree
		CdmSessionDataRepository.getDefault().addTaxon(newAcceptedTaxon);

		// Open editor for new taxon
		new OpenTaxonEditorAction(newAcceptedTaxon).run();

		// Remove old taxon from recent names list
		RecentNamesView.addRecentName(taxon);
		
		return Status.OK_STATUS;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		logger.warn(this.getLabel() + ": Redo not yet implemented.");
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		logger.warn(this.getLabel() + ": Undo not yet implemented.");
		return null;
	}
}
